<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2005 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id$

/**
  * Contains the {@link Build_DHCP} class
  *
  * @author $Author$
  * @filesource
  *
  * @package    classes
  * @subpackage backend
  */

require_once('PEAR.php');
require_once(MODDIR.'/module.php');
require_once(CLASSDIR.'/ip.php');

/**
 * Builds DHCP config files and pushes them to the respective servers
 *
 * @package classes
 * @subpackage backend
 */
class Build_DHCP extends Module
{
// {{{ vars
    var $subst_from;            // host options pattern substitutions (from)
    var $subst_to;              // ~ substitutions (to)
    var $buildfile;             // DHCP config file name and path we are building to
// }}}

/* Methods for Config File Intergrity Checking */
// {{{ needsConfigFile()

    /**
      * Returns, if the module needs a config file at all. This will be used
      * by the automated config file integrity checks, for example.
      * Defaults to false.
      * Can be called statically.
      *
      * @return boolean .cfg file vital for the module to work?
      */
    function needsConfigFile()
    {
        return true;
    }
// }}}

/* Module Overloads */
// {{{ _install()

    function _install()
    {
        return true;
    }

// }}}
// {{{ _uninstall()

    function _uninstall()
    {
        return true;
    }

// }}}
// {{{ initialize()

    function initialize()
    {
        // what path are we building to?
        $this->buildfile = BUILD .'/'. $this->config['build_dhcp']['build_file'];
        
        // save pattern substitutions for _substitute_host_options()
        $substitutions = array(
            '/%h/' => $host->name,
            '/%i/' => (IP::int2ip($host->ip)),
            '/%m/' => ($this->_convert_ha($host->ha)),
            '/%6m/' => (substr($host->ha, 0, 6))
        );

        $this->subst_from = array_keys($substitutions);
        $this->subst_to = array_values($substitutions);
    }

// }}}
// {{{ register_hooks()
    /**
      * Register module hooks
      */
    function register_hooks()
    {
        Module::add_action('backend_build', 'buildDHCP');
        Module::add_action('backend_push', 'pushDHCP');
    }
// }}}

/* Private Module Functions */
// {{{ buildDHCP()
/**
 * builds Maintain DHCP configs
 * @return boolean success
 * @access private
 */

    function buildDHCP()
    {
        // make a copy of the old config file that we can diff later
        if (file_exists($this->buildfile)) {
            if (!copy($this->buildfile, "{$this->buildfile}.bak"))
                PEAR::raiseError('Build_DHCP could not duplicate dhcp config file. Check write permissions on build dir!');
        }

        if (!$fh = @fopen($this->buildfile, 'w')) {     // try to open our config file for write access
            PEAR::raiseError("Build_DHCP does not have write access to {$this->buildfile}!");
            return false;
        }

        // include header template
        if (!$this->_include_file($fh, $this->_path . '/' . $this->config['build_dhcp']['header_file'])) {
            PEAR::raiseError(_("Build_DHCP could not open header file for inclusion: ")." {$this->config['build_dhcp']['header_file']}!");
        }

        // print global DHCP options
        fwrite($fh, "\n# global DHCP options\n");
        $globalopts = $this->_get_dhcp_options('global');
        $this->_fwrite_dhcp_options($fh, $globalopts);
        fwrite($fh, "# end global options\n\n");
        
        // build dynamic IP ranges
        $this->_print_dynamic_ranges($fh);
        
        // here used to be a class declaration of "all-hosts"
        // which included all MACs known to Maintain.
        // since all known hosts get a host{} directive, we now use
        // deny unknown-clients; instead in the subnet declarations
        
        // build subnet declarations etc.
        $this->_print_subnets($fh);
        
        // build workgroup declarations
        $this->_print_workgroups($fh);
        
        // build declarations for workgroupless hosts
        $this->_print_workgroupless_hosts($fh);
        
        // include footer template
        if (!$this->_include_file($fh, $this->_path . '/' . $this->config['build_dhcp']['footer_file'])) {
            PEAR::raiseError(_("Build_DHCP could not open footer file for inclusion: ")." {$this->config['build_dhcp']['footer_file']}!");
        }

        fclose($fh);
    }
// }}}
// {{{ pushDHCP()
/**
 * pushes DHCP configs to the server(s)
 * @return boolean success
 * @access private
 */

    function pushDHCP()
    {
        global $mode;
    
        // only push to servers if configured to do so
        if (!$this->config['push_dhcp']['push']) return true;
        
        // if update is forced, file is pushed no matter what.
        // otherwise, tests are performed.
        if (!$mode['force_update']) {
            // check if config file has changed and...
            system("diff {$this->buildfile} {$this->buildfile}.bak > /dev/null", $checkresult);
            // exit if nothing has changed
            if ($checkresult == 0) return true;

            // check conf file syntax if configured
            if ($this->config['push_dhcp']['check_syntax']) $this->_check_conf_syntax($this->buildfile);
        }
        
        // now push to all configured servers.
        
        // fetch all configured servers
        $config_keys = array_keys($this->config['push_dhcp']);
        $dhcp_servers = preg_grep("/^dhcp_server_.*$/", $config_keys);
        
        foreach ($dhcp_servers as $server_id) {
            // extract unique identifier out of server_id string
            $server_id = preg_replace("/^(dhcp_server_)/", '', $server_id);

            // build our rsync command
            $rsync_extra = (isset($this->config['push_dhcp']["dhcp_rsync_extra_{$server_id}"]) ? 
                    $this->config['push_dhcp']["dhcp_rsync_extra_{$server_id}"] : '');
            $rsync = "rsync -a ";
            $rsync .= $rsync_extra;
            
            // build different copy commands
            $copies[] = $this->buildfile ." ". $this->config['push_dhcp']["dhcp_server_{$server_id}"] .'/';

            // now try to push every file
            foreach ($copies as $push_file) {
                exec($rsync .' '. $push_file .' 2>&1', $output, $exitcode);
                if ($exitcode != 0) {
                    PEAR::raiseError(_("PUSH_DHCP an error occured when pushing configs to ").$role._(" DHCP server:"));
                    foreach ($output as $line) PEAR::raiseError($line);
                    
                    // stop pushing, return error
                    return false;
                }
            }
        } // foreach dhcp_server

        // execute post-push command (for restarting services and such)
        if ($this->config['push_dhcp']['post_push']) {
            exec($this->config['push_dhcp']['post_push'].' 2>&1', $output, $result);
            if ($result != 0) {
                PEAR::raiseError( _("Push_DHCP Error while executing post-push scripts."));
                foreach ($output as $line) PEAR::raiseError($line);
            }
        }

    }
// }}}

// {{{ _print_dynamic_ranges()
/**
 * prints all dynamic ranges
 * 
 * @param filehandle file to be written to (by reference)
 * @return boolean success
 * @access private
 */
    function _print_dynamic_ranges(&$fh)
    {
        if (!isset($fh)) return false;  // we need a file handle to write our output somewhere
        
        // build the classes for the ranges that are dynamic (even if they don't
        // have hosts associated with them.
        $dynranges = DB_DataObject::Factory('ranges');
        $zoneranges = DB_DataObject::Factory('zone_range');
        $dynranges->enabled = '1';                          // only build enabled ranges
        $dynranges->joinAdd($zoneranges, 'INNER', 'zone');  // join zones and ranges together
        $dynranges->selectAdd();                            // only select ...
        $dynranges->selectAdd("ranges.id AS rangeid, zone.zone AS zoneid");      // ... range and zone ids.
        $dynranges->orderBy('zoneid, rangeid');
        
        fwrite($fh, "# class defs for dynamic ranges\n");
            
        $dynranges->find();
        while ($dynranges->fetch()) {
            // fetch information on the current zone and range
            $zone = DB_DataObject::Factory('zone');
            $zone->id = $dynranges->zoneid;
            if ($zone->find(true) === 0) {
                PEAR::raiseError(_("Build_DHCP Zone ID {$dynranges->rangeid} not found in zone table! Skipping."));
                next;
            }

            $range = DB_DataObject::Factory('ranges');
            $range->id = $dynranges->rangeid;
            if ($range->find(true) === 0) {
                PEAR::raiseError(_("Build_DHCP Range ID {$dynranges->rangeid} not found in ranges table! Skipping."));
                next;
            }
            
            // convert zone ranges to IPv4 format, free results afterwards
            $rangestart = IP::int2ip($range->start);
            $rangeend = IP::int2ip($range->end);
            $range->free();

            // write class data to file
            fwrite($fh, "# {$zone->name} for $rangestart to $rangeend\n");
            fwrite($fh, "class \"{$zone->name}:{$rangestart}:{$rangeend}\" {\n");
            fwrite($fh, "\tmatch hardware; \n");
            fwrite($fh, "}\n");

            // write hosts in range
            fwrite($fh, "# Hosts for {$zone->name}\n");
            $hosts = DB_DataObject::Factory('host');
            $hosts->dynamic_range = $range->id;
            $hosts->zone = $zone->id;
            // only show hosts that have a hardware address configured
            $hosts->whereAdd('ha != 0');

            $hosts->find();
            while ($hosts->fetch()) {
                $ha = $this->_convert_ha($hosts->ha);   // add colons to the MAC address
                fwrite($fh, "subclass \"{$zone->name}:{$rangestart}:{$rangeend}\" 1:$ha;\n");
            }
            $zone->free();
            $hosts->free();
            
            fwrite($fh, "# end {$zone->name}\n\n");
        }
        $dynranges->free();
        $zoneranges->free();
        unset($dynranges, $zoneranges);

        return true;
    }
// }}}
// {{{ _print_subnets()
/**
 * prints subnet declarations as well as options and dynamic ranges
 * 
 * @param file handle file to be written to (by reference)
 * @return boolean success
 * @access private
 */
    function _print_subnets(&$fh)
    {
        if (!isset($fh)) return false;  // we need a file handle to write our output somewhere
        
        // build subnet declarations with their associated options
        fwrite($fh, "# subnet declarations and options\n");
        
        // get all vlans for which subnets exist
        $vlans = DB_DataObject::Factory('subnet');
        $vlans->selectAdd();
        $vlans->selectAdd('DISTINCT vlan, vlan.name');
        $vlans->orderBy('vlan');
        $vlans->whereAdd();
        $vlans->whereAdd('vlan != 0');
        $vlans->whereAdd("status != 'hidden'");

        $vlannames = DB_DataObject::Factory('vlan');
        $vlans->joinAdd($vlannames);
        
        $vlans->find();

        fwrite($fh, "# subnets which belong to vlans\n");
        while($vlans->fetch()) {
            fwrite($fh, 'shared-network "'.str_replace('"', '', $vlans->name).'" {'."\n");
            $this->_print_subnets_for_vlan($vlans->vlan, $fh, "\t");
            fwrite($fh, "}\n");
        }

        // get all subnets which don't belong to a vlan
        // (i.e. the vlan id is 0)
        fwrite($fh, "# subnets without vlans\n");
        $this->_print_subnets_for_vlan(0, $fh, "");
        
        fwrite($fh, "# end of vlan and subnet declarations\n");
        
        return true;
    }
// }}}
// {{{ _print_workgroups()
/**
 * prints workgroup declarations as well as options and hosts
 * 
 * @param filehandle file to be written to (by reference)
 * @return boolean success
 * @access private
 */
    function _print_workgroups(&$fh)
    {
        if (!isset($fh)) return false;  // we need a file handle to write our output somewhere
        
        // build workgroup declarations with their associated options
        fwrite($fh, "# workgroup declarations\n");
        
        $workgroups = DB_DataObject::Factory('workgroup');
        $workgroups->orderBy('id');
        
        $workgroups->find();
        while ($workgroups->fetch()) {
            fwrite($fh, "group { # {$workgroups->name}\n");
            
            // print DHCP options for workgroup
            $wgopts = $this->_get_dhcp_options('workgroup', $workgroups->id);
            $this->_fwrite_dhcp_options($fh, $wgopts, "\t");

            // now get all hosts with a known MAC address for this workgroup
            // and print them all out.
            
            // first fetch the group dhcp options for this group
            // which have 'host' as their scope
            $wghostopts = $this->_get_dhcp_options('workgroup', $workgroups->id, 'host');
            
            $hosts = DB_DataObject::Factory('host');
            $hosts->workgroup = $workgroups->id;
            $hosts->whereAdd("ha != '000000000000'");
            $hosts->whereAdd("CHAR_LENGTH(ha) = 12");
            $hosts->enabled = 1;
            $hosts->orderBy('ip');
            $domains = DB_DataObject::Factory('domain');
            $hosts->joinAdd($domains);
            $hosts->selectAdd();
            $hosts->selectAdd("host.id, host.name, host.ip, host.ha, host.type, domain.name AS domain");

            $hosts->find();
            while ($hosts->fetch()) {
                fwrite($fh, "\thost {$hosts->name}.{$hosts->domain} {\n");
                // is this a dynamic host?
                // if not, add fixed IP
                if ($hosts->ip != 0) {
                    fwrite($fh, "\t\tfixed-address " . (IP::int2ip($hosts->ip)) . ";\n");
                }
                
                $ha = $this->_convert_ha($hosts->ha);   // add colons to the MAC address
                fwrite($fh, "\t\thardware ethernet $ha;\n");

                // get host-specific dhcp options
                $hostopts = $this->_get_dhcp_options('host', $hosts->id);

                // now do pattern substitutions on host options and
                // write them to the conf file afterwards
                $this->_fwrite_dhcp_options($fh, $this->_substitute_host_options($wghostopts, $hosts), "\t\t");
                $this->_fwrite_dhcp_options($fh, $this->_substitute_host_options($hostopts, $hosts), "\t\t");

                unset($hostopts);
                
                fwrite($fh, "\t}\n");
            }
            $hosts->free();
            $domains->free();

            fwrite($fh, "}\n");
            // echo memory_get_usage()."\n";    // debugging
        }
        $workgroups->free();
        fwrite($fh, "# end workgroups and their hosts\n\n");

        return true;
    }
// }}}
// {{{ _print_workgroupless_hosts()
/**
 * prints host{} declarations for all hosts that not belong to a workgroup
 * 
 * @param filehandle file to be written to (by reference)
 * @return boolean success
 * @access private
 */
    function _print_workgroupless_hosts(&$fh)
    {
        if (!isset($fh)) return false;  // we need a file handle to write our output somewhere
        
        // build workgroup declarations with their associated options
        fwrite($fh, "# workgroupless hosts\n");
        
        $hosts = DB_DataObject::Factory('host');
        $hosts->whereAdd("(workgroup IS NULL OR workgroup = 0)");
        $hosts->whereAdd("ha != '000000000000'");
        $hosts->whereAdd("CHAR_LENGTH(ha) = 12");
        $hosts->enabled = 1;
        $hosts->orderBy('ip');
        $domains = DB_DataObject::Factory('domain');
        $hosts->joinAdd($domains);
        $hosts->selectAdd();
        $hosts->selectAdd("host.id, host.name, host.ip, host.ha, host.type, domain.name AS domain");

        $hosts->find();
        while ($hosts->fetch()) {
            fwrite($fh, "host {$hosts->name}.{$hosts->domain} {\n");
            // is this a dynamic host? if not, add fixed IP
            if ($hosts->ip != 0) {
                fwrite($fh, "\tfixed-address " . (IP::int2ip($hosts->ip)) . ";\n");
            }
            
            $ha = $this->_convert_ha($hosts->ha);   // add colons to the MAC address
            fwrite($fh, "\thardware ethernet $ha;\n");

            // get host-specific dhcp options
            $hostopts = $this->_get_dhcp_options('host', $hosts->id);

            // now do pattern substitutions on host options and
            // write them to the conf file afterwards
            $this->_fwrite_dhcp_options($fh, $this->_substitute_host_options($hostopts, $hosts), "\t");

            unset($hostopts);
            
            fwrite($fh, "\t}\n");
        }
        $hosts->free();
        $domains->free();

        fwrite($fh, "# end workgroupless hosts\n");

        return true;
    }
// }}}

// {{{ _print_subnets_for_vlan
    /**
      * print the subnet declarations for a given vlan number to the
      * file specified by the file handle
      *
      * @private
      * @param vlan id (DB primary key, not assigned id)
      * @param file handle
      * @param prefix (indentation, e.g. \t\t)
      * @return void
      */
    function _print_subnets_for_vlan($vlan, &$fh, $prefix='')
    {
        $subnets = DB_DataObject::Factory('subnet');
        $subnets->status = 'visible';
        $subnets->orderBy('id');
        $subnets->vlan = $vlan;

        $subnets->find();
        while ($subnets->fetch()) {
            $subnet = IP::int2ip($subnets->subnet);
            $netmask = IP::int2ip($subnets->netmask);
            
            fwrite($fh, "{$prefix}subnet $subnet netmask $netmask {\n");
            
            // get and print DHCP options for subnet to file
            $subnetopts = $this->_get_dhcp_options('subnet', $subnets->id);
            $this->_fwrite_dhcp_options($fh, $subnetopts, "{$prefix}\t");
            
            // add dynamic ranges for this subnet
            $ranges = DB_DataObject::Factory('ranges');
            $ranges->selectAdd();
            $ranges->selectAdd("id, start, end, allow_all_hosts");
            $ranges->whereAdd();
            $ranges->whereAdd("type = 'dynamic'");
            $ranges->enabled = 1;
            $ranges->subnet = $subnets->id;
            $ranges->find();

            while ($ranges->fetch()) {
                $rangestart = IP::int2ip($ranges->start);
                $rangeend = IP::int2ip($ranges->end);
                
                fwrite($fh, "{$prefix}\tpool {\n");
                // TODO handle DHCP failover
                //fwrite($fh, "{$prefix}\t\tfailover peer \"dhcp\";\n");
                
                $zonerange = DB_DataObject::Factory('zone_range');
                $zonerange->range = $ranges->id;
                $zone = DB_DataObject::Factory('zone'); // join with zones for name
                $zone->selectAdd();
                $zone->selectAdd('name');
                $zonerange->joinAdd($zone);
                $zonerange->find();
                while ($zonerange->fetch()) {
                    fwrite($fh, "{$prefix}\t\tallow members of \"{$zonerange->name}:$rangestart:$rangeend\";\n");
                }
                $zonerange->free();
                $zone->free();
                fwrite($fh, "{$prefix}\t\trange $rangestart $rangeend;\n");

                if ($ranges->allow_all_hosts) {
                    fwrite($fh, "{$prefix}\t\tdeny unknown-clients;\n");
                }
                
                fwrite($fh, "{$prefix}\t}\n");
            }
            $ranges->free();
            
            fwrite($fh, "{$prefix}}\n\n");
        }
        $subnets->free();
    }
// }}}
// {{{ _get_dhcp_options()
/**
 * get DHCP option values for given type and scope
 *
 * @param string type of group (subnet or workgroup)
 * @param int id of subnet or workgroup for which options are to be printed
 *          (defaults to -1 which will be interpreted as 'no restriction')
 * @param string scope (defaults to $type)
 * @return array options
 * @access private
 */

    function _get_dhcp_options($type, $id = -1, $scope = '')
    {
        // get options, joined with their names
        $options = DB_DataObject::Factory('object_option');
        $options->type = $type;
        //$options->whereAdd("(scope='$type' OR scope='host')");
        $options->scope = ($scope == '' ? $type : $scope);
        if ($id != -1)
            $options->object_id = $id;
        $optionnames = DB_DataObject::Factory('dhcp_options');
        $options->joinAdd($optionnames, 'INNER', 'do');
        $options->selectAdd();
        $options->selectAdd('do.type AS optiontype, do.name AS optionname, object_option.value AS optionvalue');
        $options->find();
        
        $optarray = array();
        while ($options->fetch()) {
            array_push ($optarray,
                array('type' => $options->optiontype,
                'name' => $options->optionname,
                'value' => $options->optionvalue));
        }
        $optionnames->free();
        $options->free();
        unset($optionnames, $options);
        
        return $optarray;
    }
// }}}
// {{{ _fwrite_dhcp_options()
/**
 * write DHCP options to file that have been written to an array before by 
 * _print_dhcp_options or _print_workgroups
 * option array format is [number][options] with number index no and
 * option = type|name|value
 *
 * @param fh Filehandle to output file
 * @param options array holding the options to be written
 * @param offset String to be printed in front of option line, default empty 
 * @return success
 * @access private
 */
    function _fwrite_dhcp_options(&$fh, &$options, $offset = '')
    {
        foreach ($options as $opt) {
            switch ($opt['type']) {
            case 'option':
                fwrite($fh, $offset . "option {$opt['name']} {$opt['value']};\n");
                break;

            case 'quoted-option':
                fwrite($fh, $offset . "option {$opt['name']} \"{$opt['value']}\";\n");
                break;

            case 'quoted':
                fwrite($fh, $offset . "{$opt['name']} \"{$opt['value']}\";\n");
                break;

            default:
                fwrite($fh, $offset . "{$opt['name']} {$opt['value']};\n");
                break;
            }
        }

        return true;
    }
// }}}
// {{{ _substitute_host_options()
/**
 * given a host option array, it does some pattern substitution
 * (for a list of substitutions see the constructor)
 *
 * @param options option array with host options
 * @param hosts DB_DataObject currently pointing to the current host (used for pattern substitution) and not to be altered herein
 * @return array substituted options
 * @access private
 */
    function _substitute_host_options(&$options, &$host)
    {
        // pattern substitutions set in the constructor
        foreach($options as $optnr => $opt) {
            $options[$optnr]['value'] = preg_replace($this->subst_from, $this->subst_to, $opt['value']);
        }

        return $options;
    }
// }}}

// {{{ check_current_config()
/**
 * publicly accessible config to check current config file.
 * is used by /bin/build/check_dhcp
 *
 * @return boolean errors occured?
 * @access private
 */
    function check_current_config()
    {
        return $this->_check_conf_syntax($this->buildfile);
    }
// }}}
// {{{ _check_conf_syntax()
/**
 * checks the syntax of a given configuration file.
 * A dhcpd3 server has to be installed locally therefore.
 *
 * @param String config file path
 * @return boolean errors occured?
 * @access private
 */
    function _check_conf_syntax($configfile)
    {
        // $mode comes from bin/build/all and will tell us if debug is enabled
        global $mode;
        
        // sorry, currently we have to be running as root
        // otherwise dhcpd3 will have a permission error
        $processUser = posix_getpwuid(posix_geteuid());
        if ($processUser['name'] != 'root') {
            PEAR::raiseError(_('Build_DHCP DHCP syntax check can only be executed as root! Aborting DHCP syntax check.'));
            return false;
        }

        $dhcpd3 = trim(`which dhcpd3`);    // last char is a newline!
        if (!is_executable($dhcpd3)) {
            PEAR::raiseError(_('Build_DHCP dhcpd3 could not be found or is not accessible! Aborting DHCP syntax check.'));
            return false;
        }
        
        // if in debug mode, print all error messages to stderr
        if ($mode['debug'] === true) {
            $check_result = system("$dhcpd3 -t -cf $configfile 2>&1 1>/dev/null");
        } else {
            $check_result = exec("$dhcpd3 -t -cf $configfile 2>&1 1>/dev/null");
        }
        
        // errors occured?
        $okay = ereg("/(exiting|duplicate entries)/", $check_result);
        
        if (!$okay) {
            // notify the admins of the error
            $errormsg = (_('Building the DHCP config file failed. The file contains errors and will not be pushed to the servers. Please check this with bin/build/check-dhcp ASAP!', 'Build_DHCP'));
            Email::adminEmail($errormsg, 'Build_DHCP');
            PEAR::raiseError($errormsg);
            
            return false;
        } else {
            return true;
        }
    }
// }}}

// {{{ _push_to_server()
/**
 * pushes configs to a specific server, specified through its role
 *
 * @param string role: 'primary' or 'secondary'
 * @return boolean success
 * @access private
 */
    function _push_to_server($role)
    {
        if ($role != 'primary' && $role != 'secondary') return false;
        // do not do anything if server string is empty
        if ($this->config['push_dhcp'][$role] == '') return true;


        return true;
    }
// }}}

// {{{ _convert_ha()
/**
 * adds colons to hardware address from database
 *
 * @param string hardware address without colons (12 characters)
 * @return string hardware address with colons (17 characters)
 * @access private
 */
    function _convert_ha($ha)
    {
        return substr(chunk_split($ha, 2, ':'), 0, -1);
    }
// }}}
// {{{ _include_file
/**
 * include a template file into the output data file
 *
 * @param filehandle of output file
 * @param string filename of file to be included
 * @return boolean success
 * @access private
 */
    function _include_file(&$fh, $incfile)
    {
        // try to open include template and put it into the config file
        if (!$inc_fh = @fopen($incfile, 'r')) {
            return false;
        } else {
            while (!feof($inc_fh)) {
                fwrite($fh, fgets($inc_fh));
            }
            fclose($inc_fh);
            return true;
        }

    }
// }}}
}

?>
